﻿using UnityEngine;
using UnityEngine.SceneManagement;
using System.Collections;
using System.Collections.Generic;
using DG.Tweening;

public class GameManager : MonoBehaviour
{
	public GameObject player1;
	public GameObject player2;
	public GameObject logo;

	public List<KeyCode> currentDance = new List<KeyCode> ();
	public List<KeyCode> lastDance = new List<KeyCode> ();

	public AudioClip[] cowSounds;

	public GameObject arrowPrefab;
	public Sprite[] arrows;

	private Dictionary<GameObject, PlayerValues> playerValues;
	private KeyCode playerInput;
	private bool allowAI = true;
	private bool useAI = false;

	void Start ()
	{
		playerValues = new Dictionary<GameObject, PlayerValues> ();
		playerValues [player1] = player1.GetComponent<PlayerValues> ();
		playerValues [player2] = player2.GetComponent<PlayerValues> ();

		StartCoroutine (GameLoop ());
	}

	private IEnumerator GameLoop ()
	{
		yield return StartCoroutine (RoundStarting ());
		yield return StartCoroutine (RoundPlaying ());
		yield return StartCoroutine (RoundEnding ());
	}

	private IEnumerator RoundStarting ()
	{
		player1.transform.localPosition = new Vector3 (-15f, player1.transform.localPosition.y);
		player1.transform.DOLocalMoveX (-6.7f, 2f);

		player2.transform.localPosition = new Vector3 (12f, player2.transform.localPosition.y);
		player2.transform.DOLocalMoveX (4.0f, 2f);
		yield return null;
	}

	private IEnumerator RoundPlaying ()
	{
		float speed = 2;
		while (true) {
			float delay = (lastDance.Count > 0) ? 1f : 0f;

			// player 1
			player1.transform.DOScale (0.4f, speed).SetDelay (delay);
			player2.transform.DOScale (0.25f, speed).SetDelay (delay);
			player1.transform.DOLocalMoveY (-1.6f, speed).SetDelay (delay);
			player2.transform.DOLocalMoveY (-0.6f, speed).SetDelay (delay);
			yield return new WaitForSeconds (1f);
			logo.transform.localPosition = new Vector3 (-7f, 10f);
			logo.transform.DOLocalMoveY (2.5f, 1f);

			yield return StartCoroutine (PlayerDancing (player1));
			if (playerValues [player1].power <= 0) {
				yield return StartCoroutine (PlayerLost (player1));
				break;
			}

			// player 2
			player1.transform.DOScale (0.25f, speed).SetDelay (1f);
			player2.transform.DOScale (0.4f, speed).SetDelay (1f);
			player1.transform.DOLocalMoveY (-0.6f, speed).SetDelay (1f);
			player2.transform.DOLocalMoveY (-1.6f, speed).SetDelay (1f);
			yield return new WaitForSeconds (1f);
			logo.transform.localPosition = new Vector3 (7f, 10f);
			logo.transform.DOLocalMoveY (2.5f, 1f);

			yield return StartCoroutine (PlayerDancing (player2));
			if (playerValues [player2].power <= 0) {
				yield return StartCoroutine (PlayerLost (player2));
				break;
			}
		}
	}

	private IEnumerator RoundEnding ()
	{
		yield return new WaitForSeconds (1.0f);
	}

	private IEnumerator PlayerDancing (GameObject player)
	{
		while (!isSequenceDone ()) {
			yield return StartCoroutine (readPlayerMove (player));
			yield return StartCoroutine (handlePlayerMove (player));

			if (lastDance.Count > 0 && !isKeyValid ()) {
				playerValues [player].power = 0;
				break;
			}
		}

		// add new element
		lastDance.Add (currentDance [currentDance.Count - 1]);
		currentDance = new List<KeyCode> ();
		GameObject.Find ("CloudGenerator").GetComponent<CloudGenerator> ().sequenceLength = lastDance.Count;
	}

	private bool isKeyValid ()
	{
		return (currentDance.Count > lastDance.Count || lastDance [currentDance.Count - 1] == currentDance [currentDance.Count - 1]);
	}

	private bool isSequenceDone ()
	{
		return currentDance.Count == lastDance.Count + 1;
	}

	private IEnumerator PlayerLost (GameObject player)
	{
		Debug.Log (player.name + " Lost");
		player.transform.Find ("Lightning").GetComponent<SpriteRenderer> ().enabled = true;

		yield return new WaitForSeconds (0.2f);

		// enable struck cow, disable normal cow
		player.transform.Find ("Body").GetComponent<SpriteRenderer> ().enabled = false;
		player.transform.Find ("CowStruck").GetComponent<SpriteRenderer> ().enabled = true;
		player.transform.Find ("CowStruck").GetComponent<AudioSource> ().PlayScheduled (0);

		yield return new WaitForSeconds (0.2f);

		player.transform.Find ("Lightning").GetComponent<SpriteRenderer> ().enabled = false;

		// wait a bit
		yield return new WaitForSeconds (3.0f);

		// revert
		SceneManager.LoadScene (0);
	}

	private IEnumerator readPlayerMove (GameObject player)
	{
		float aiTakeOverPoint = Time.time + 5f;

		while (true) {
			if (allowAI && player == player2 && !useAI && Time.time > aiTakeOverPoint) {
				useAI = true;
				Debug.Log ("Switching to AI");
			}

			if (player == player2 && useAI) {
				logo.transform.DOLocalMoveY (10f, 0.5f);
				if (currentDance.Count < lastDance.Count) {
					playerInput = lastDance [currentDance.Count];
				} else {
					switch (Random.Range (0, 3)) {
					case 0:
						playerInput = KeyCode.LeftArrow;
						break;
					case 1:
						playerInput = KeyCode.RightArrow;
						break;
					case 2:
						playerInput = KeyCode.UpArrow;
						break;
					case 3:
						playerInput = KeyCode.DownArrow;
						break;
					}
				}
				yield return new WaitForSeconds (1.0f);
				break;
			} else {
				if (Input.anyKeyDown) {
					if (player == player2) {
						allowAI = false;
					}
					logo.transform.DOLocalMoveY (10f, 0.5f);
				}

				if (Input.GetKeyDown (KeyCode.Escape)) {
					Application.Quit ();
				} else if (Input.GetKeyDown (KeyCode.LeftArrow)) {
					playerInput = KeyCode.LeftArrow;
					break;
				} else if (Input.GetKeyDown (KeyCode.RightArrow)) {
					playerInput = KeyCode.RightArrow;
					break;
				} else if (Input.GetKeyDown (KeyCode.UpArrow)) {
					playerInput = KeyCode.UpArrow;
					break;
				} else if (Input.GetKeyDown (KeyCode.DownArrow)) {
					playerInput = KeyCode.DownArrow;
					break;
				}
			}
			yield return null;
		}
	}

	private IEnumerator handlePlayerMove (GameObject player)
	{
		currentDance.Add (playerInput);
		string animation = "";
		int audioIdx = 0;
		ArrowAnimation.Direction direction = ArrowAnimation.Direction.Down;

		switch (playerInput) {
		case KeyCode.LeftArrow:
			animation = "DanceLeft";
			audioIdx = 0;
			direction = ArrowAnimation.Direction.Left;
			break;
		case KeyCode.RightArrow:
			animation = "DanceRight";
			audioIdx = 1;
			direction = ArrowAnimation.Direction.Right;
			break;
		case KeyCode.UpArrow:
			animation = "DanceUp";
			audioIdx = 2;
			direction = ArrowAnimation.Direction.Up;
			break;
		case KeyCode.DownArrow:
			animation = "DanceDown";
			audioIdx = 3;
			direction = ArrowAnimation.Direction.Down;
			break;
		}
		player.GetComponentInChildren<Animator> ().SetTrigger (animation);
		player.GetComponent<AudioSource> ().PlayOneShot (cowSounds [audioIdx]);

		GameObject arrow = Instantiate (arrowPrefab);
		arrow.GetComponent<SpriteRenderer> ().sprite = arrows [audioIdx];
		arrow.transform.position = new Vector3 (player.transform.position.x + 1.5f, player.transform.position.y, player.transform.position.z);
		arrow.GetComponent<ArrowAnimation> ().direction = direction;

		yield return null;
	}
}