﻿using UnityEngine;
using UnityEngine.Networking;
using UnityEngine.Networking.Match;
using UnityEngine.Networking.Types;
using System.Collections;

public class MatchMaker : NetworkBehaviour {
	NetworkMatch networkMatch;

	bool matchCreated = false;
	NetworkID networkId = NetworkID.Invalid;

	[SyncVar]
	public bool matched = false;

	void Awake() {
		networkMatch = gameObject.AddComponent<NetworkMatch> ();
	}

	void Start () {
		listMatches ();
	}

	void listMatches() {
		networkMatch.ListMatches (0, 100, "", OnMatchListReceived);
	}

	void createMatch() {
		int id = Random.Range (0, 99999999);
		networkMatch.CreateMatch ("Game" + id, 2, true, "", OnMatchCreated);
	}

	void OnMatchCreated(CreateMatchResponse response) {
		Debug.Log ("Creation Success: " + response.success);
		if (response.success) {
			matchCreated = true;
			networkId = response.networkId;
		}
	}

	void OnMatchListReceived(ListMatchResponse response) {
		Debug.Log ("List Success: " + response.success);
		Debug.Log ("List Matches: " + response.matches.Count);

		if (response.success && response.matches != null) {
			foreach (MatchDesc match in response.matches) {
				// don't join our own game
				if (match.currentSize < 2 && match.networkId != networkId) {
					Debug.Log ("Joining Match: " + match.networkId);
					matched = true;
					networkMatch.JoinMatch (match.networkId, "", OnMatchJoined);
					break;
				}
			}
		}

		if (!matched) {
			// nothing found, create one ourselves
			if (!matchCreated) {
				createMatch ();
			}
			listMatches ();
		}
	}

	void OnMatchJoined(JoinMatchResponse response) {
		Debug.Log ("Matched: " + response.success);
		if (response.success) {
			matched = true;
		}
	}
}
